/*
	$Id: SoftwareTransfer.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Häkkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Software;
import net.sf.basedb.core.SoftwareType;
import net.sf.basedb.core.SystemItems;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
This class provides methods to transfer Software data from base 1 to BASE 2.

@author Gregory
@version 2.0
@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/

public class SoftwareTransfer extends CommonItemTransfer
{
	/**
	All software from BASE 1 are of type FEATURE_EXTRACTION
	 */
	private int softwareTypeId = 0;
	
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.SoftwareTransfer");

	/**
	@param userT <code>UserTransfer</code>
	@param groupT <code>GroupTransfer</code>
	 */
	SoftwareTransfer(UserTransfer userT, GroupTransfer groupT)
	{
		super(userT, groupT);
	}

	public void start()
	{
		log.info("Transferring Softwares...");
		softwareTypeId = SystemItems.getId(SoftwareType.FEATURE_EXTRACTION);
		int count = getInt("countSoftware");
		initIdMap(count);
		startProgress(count, "Softwares");
		run(prepareStatementFromFile("selectSoftware"), 0, getBatchSize());
		done();
		log.info("Transferring Softwares...DONE");
	}

	/**
	Returns a Software from the current row in a given ResultSet.
	
	@param b2c {@link DbControl}
	@param rs {@link ResultSet}
	@return Software
	 */
	Software createItem(DbControl b2c, ResultSet rs) throws BaseException,
			SQLException
	{
		Software software = SoftwareType.getById(b2c, softwareTypeId).newSoftware();
		setCommonItemData(b2c, (CommonItem) software, rs);
		software.setVersionString(rs.getString(9));
		if (log.isDebugEnabled())
		{
			log.debug("Transfered Software: " + software.getName());
			log.debug("--------------------------------------------------------------------------");
		}
		return software;
	}
	
	/**
	Returns a Software by the given base 1 software id.
	
	@param dbc <code>DbControl</code> used to get the Software.
	@param base1SoftwareId
	@return <code>Software</code> or null if the given software id does not exist.
	 */
	public Software getSoftwareById(DbControl dbc, int base1SoftwareId)
	{
		try
		{
			Integer base2Id = getBase2Id(base1SoftwareId);
			return base2Id == null ? null : Software.getById(dbc, base2Id);
		}
		catch (BaseException e)
		{
			log.warn("Couldn't get software in BASE1 by id: " + base1SoftwareId);
			if (log.isDebugEnabled()) log.debug("\t" + e);
		}
		return null;
	}
	
}
