/**
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import java.util.ArrayList;
import java.util.List;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.FileStoreEnabled;
import net.sf.basedb.core.PermissionDeniedException;

/**	
	Data loader implementation that load the files in a file set
	in {@link FileStoreEnabled} items.
	
	@author Nicklas
	@version 2.16
	@base.modified $Date: 2010-08-05 14:29:44 +0200 (Thu, 05 Aug 2010) $
*/
public class FileSetMemberLoader 
	implements DataLoader<FileStoreEnabled>
{

	public FileSetMemberLoader()
	{}
	
	@Override
	public Object getData(ExportedProperty exportedProperty, FileStoreEnabled item) 
	{
		if (!item.hasFileSet()) return null;
		
		DbControl dc = ((BasicItem)item).getDbControl();
		List<FileSetMember> members = item.getFileSet().getMembers().list(dc);
		List<File> files = new ArrayList<File>(members.size());
		for (FileSetMember member : members)
		{
			try
			{
				files.add(member.getFile());
			}
			catch (PermissionDeniedException ex)
			{}
		}
		return files;
	}

}
