/*
	$Id: SystemItem.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.SystemData;

/**
	A <code>SystemItem</code> is an item which has an
	additional systemwide unique id in the form of string. 
	A system id is required when we need to make sure that we can 
	get a specific item without knowing the numeric id. Example of 
	such items are the root user and the everyone group.
	<p>
	Items with a system id can only be created during installation
	and cannot be removed.
	<p>
	
	<b>Reference implementation</b><br>
	<pre class="code">
public String getSystemId()
{
   return getData().getSystemId();
}
public boolean isSystemItem()
{
   return getSystemId() != null;
}
</pre>

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public interface SystemItem
	extends Identifiable
{
	/**
		The maximum length of the system ID that can be stored in the database.
		@see #getSystemId()
	*/
	public static final int MAX_SYSTEM_ID_LENGTH = 
		SystemData.MAX_SYSTEM_ID_LENGTH;
	
	
	/**
		Get the system id for the item.
		@return The id of the item or null if it is not a system item
	*/
	public String getSystemId();
	
	/**
		Check if the item is a system item or not. A system item have a non-null
		value for the system id.
		@return TRUE if this item is a system item, FALSE otherwise
	*/
	public boolean isSystemItem();

}
