/*
	$Id: BooleanParameterValueData.java 5437 2010-10-01 11:51:57Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.List;
import java.util.ArrayList;

/**
	Boolean parameter value.

	@author Nicklas, Samuel
	@version 2.0
	@see <a href="../../../../../../html/developerdoc/api/data_api.parameters.html">Parameters overview</a>
	@hibernate.subclass discriminator-value="7"
	@base.modified $Date: 2010-10-01 13:51:57 +0200 (Fri, 01 Oct 2010) $
*/
public class BooleanParameterValueData
	extends ParameterValueData<Boolean>
{
	public BooleanParameterValueData()
	{}
	
	public BooleanParameterValueData(Boolean... values)
	{
		super(values);
	}
	
	/**
		@return 7 (Type.BOOLEAN)
		@since 2.16
	*/
	public int getValueType()
	{
		return 7;
	}
	
	private List<Boolean> values;
	/**
		@hibernate.bag table="`BooleanValues`" lazy="true" cascade="all"
		@hibernate.collection-key column="`id`"
		@hibernate.collection-element column="`value`" type="boolean" not-null="true"
	*/
	public List<Boolean> getValues()
	{
		if (values == null)
		{
			values = new ArrayList<Boolean>();
		}
		return values;
	}
	void setValues(List<Boolean> values)
	{
		this.values = values;
	}

	public String toString()
	{
		return values.toString();
	}
}
