/*
  $Id: DiskUsageData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import net.sf.basedb.core.DiskConsumable;

/**
	This class holds information about a users and/or groups disk usage.
	
	@author enell
	@version 2.0
	@see net.sf.basedb.core.DiskUsage
	@see <a href="../../../../../../html/developerdoc/api/data_api.quota.html">Quota overview</a>
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
	@hibernate.class table="`DiskUsage`" lazy="true"
*/
public class DiskUsageData
	extends BasicData
{
	
	/**
		Creates a new DiskUsageData.
	*/
	public DiskUsageData()
	{}
	
	private int itemType;
	/**
		Get the item type this annotation set is associated with.
		@hibernate.property column="`item_type`" type="int" not-null="true" update="false"
		@since 2.2
	*/
	public int getItemType()
	{
		return itemType;
	}
	public void setItemType(int itemType)
	{
		this.itemType = itemType;
	}
	
	private int location;
	/**
		Get the location of the {@link DiskConsumable} item.
		@see net.sf.basedb.core.Location
		@hibernate.property column="`location`" type="int" not-null="true"
	*/
	public int getLocation()
	{
		return location;
	}
	public void setLocation(int location)
	{
		this.location = location;
	}

	private long bytes;
	/**
		Get the number of bytes the {@link DiskConsumable}
		item is using.
		@hibernate.property column="`bytes`" type="long" not-null="true"
	*/
	public long getBytes()
	{
		return bytes;
	}
	public void setBytes(long bytes)
	{
		this.bytes = bytes;
	}
	
	private QuotaTypeData quotaType;
	/**
		Get the {@link QuotaTypeData} for this DiskUsageData.
		@hibernate.many-to-one column="`quotatype_id`" not-null="true" outer-join="false"
	*/
	public QuotaTypeData getQuotaType()
	{
		return quotaType;
	}
	public void setQuotaType(QuotaTypeData quotaType)
	{
		this.quotaType = quotaType;
	}
	
	private GroupData group;
	/**
		Get the {@link GroupData} that owns the {@link DiskConsumable} item.
		@hibernate.many-to-one column="`group_id`" not-null="false" outer-join="false"
	*/
	public GroupData getGroup()
	{
		return group;
	}
	public void setGroup(GroupData group)
	{
		this.group = group;
	}
	
	private UserData user;
	/**
		Get the user that owns the {@link DiskConsumable} item.
		@hibernate.many-to-one column="`user_id`" not-null="true" outer-join="false"
	*/
	public UserData getUser()
	{
		return user;
	}
	public void setUser(UserData user)
	{
		this.user = user;
	}
}
