/*
  $Id: ExtraValueTypeData.java 5319 2010-04-20 08:02:16Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;

/**
	This represents the definitiona of an extra value type 
	that can be calculated by a plugin.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.ExtraValueType
	@see <a href="../../../../../../html/developerdoc/api/data_api.experiments.html">Experiments and analysis overview</a>
	@hibernate.class table="`ExtraValueTypes`" lazy="true"
	@base.modified $Date: 2010-04-20 10:02:16 +0200 (Tue, 20 Apr 2010) $
*/
public class ExtraValueTypeData
	extends BasicData
	implements NameableData, RemovableData, RegisteredData
{
	public ExtraValueTypeData()
	{}
	
	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}

	private String description;
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	public boolean isRemoved()
	{
		return removed;
	}
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------
	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------
	
	/**
		The maximum length of the external id that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external id for this <code>ExtraValueType</code> item.
		@hibernate.property column="`external_id`" type="string" length="255" 
			not-null="true" unique="true"
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}

	private int valueType;
	/**
		The code for the value type.
		@hibernate.property column="`value_type`" type="int" not-null="true" update="false"
	*/
	public int getValueType()
	{
		return valueType;
	}
	public void setValueType(int valueType)
	{
		this.valueType = valueType;
	}

	private ColoringData coloring;
	/**
		Get the coloring information for this extra value type.
		@hibernate.component
	*/
	public ColoringData getColoring()
	{
		if (coloring == null) coloring = new ColoringData();
		return coloring;
	}
	public void setColoring(ColoringData coloring)
	{
		this.coloring = coloring;
	}
	
	private int averageMethod;
	/**
		The average method for thid formula: 
		<ul>
		<li>0 = none, {@link net.sf.basedb.core.Formula.AverageMethod#NONE}; 
			no average is defined for this formula
		<li>1 = geometric mean, {@link net.sf.basedb.core.Formula.AverageMethod#GEOMETRIC_MEAN}; 
			this formula uses geometric mean
		<li>2 = arithmetic mean, {@link net.sf.basedb.core.Formula.AverageMethod#ARITHMETIC_MEAN}; 
			this formula uses arithmetic mean
		<li>3 = min, {@link net.sf.basedb.core.Formula.AverageMethod#MIN}; 
			this formula uses the min value as a "mean"
		<li>4 = max, {@link net.sf.basedb.core.Formula.AverageMethod#MAX}; 
			this formula uses the max value as a "mean"
		</ul>
		
		@hibernate.property column="`average_method`" type="int" not-null="true"
		@since 2.4
	*/
	public int getAverageMethod()
	{
		return averageMethod;
	}
	public void setAverageMethod(int averageMethod)
	{
		this.averageMethod = averageMethod;
	}
	
	private Set<ExtraValueData> extraValues;
	/**
		This is the inverse end.
		@see ExtraValueData#getExtraValueType()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`extravaluetype_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ExtraValueData"
		@since 2.15
	*/
	Set<ExtraValueData> getExtraValues()
	{
		return extraValues;
	}
	void setExtraValues(Set<ExtraValueData> extraValues)
	{
		this.extraValues = extraValues;
	}

}
