/**
	$Id: FileServerData.java 5362 2010-06-16 12:28:36Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	File entries that are external files can be linked to a file server. This
	contains information about username/password and/or SSL keys that are
	required to access the server. 

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2010-06-16 14:28:36 +0200 (Wed, 16 Jun 2010) $
	@hibernate.class table="`FileServers`" lazy="true"
*/
public class FileServerData
	extends CommonData
{

	public FileServerData()
	{}
	
	public static final int MAX_USERNAME_LENGTH = 255;
	private String username;
	/**
		Get the username that is used to login to the server. If null
		no login is required.
		@hibernate.property column="`username`" type="string" length="255" not-null="false"
	*/
	public String getUsername()
	{
		return username;
	}
	public void setUsername(String username)
	{
		this.username = username;
	}

	public static final int MAX_PASSWORD_LENGTH = 255;
	private String password;
	/**
		Get the password that is used to login to the server. 
		@hibernate.property column="`password`" type="string" length="255" not-null="false"
	*/
	public String getPassword()
	{
		return password;
	}
	public void setPassword(String password)
	{
		this.password = password;
	}
	
	public static final int MAX_CERTIFICATE_LENGTH = 65535;
	private byte[] serverCertificate;
	/**
		Get the public server certificate indicating that we should trust
		https connections to the server.
		@hibernate.property column="`server_certificate`" type="binary" length="65535" not-null="false"
	*/
	public byte[] getServerCertificate()
	{
		return serverCertificate;
	}
	public void setServerCertificate(byte[] serverCertificate)
	{
		this.serverCertificate = serverCertificate;
	}
	
	private byte[] clientCertificate;
	/**
		Get the client certificate that we need to send to the server 
		to make the server accept our connection.
		@hibernate.property column="`client_certificate`" type="binary" length="65535" not-null="false"
	*/
	public byte[] getClientCertificate()
	{
		return clientCertificate;
	}
	public void setClientCertificate(byte[] clientCertificate)
	{
		this.clientCertificate = clientCertificate;
	}
	
	private String clientPassword;
	/**
		Get the password that is used to unlock the client certificate.
		@hibernate.property column="`client_password`" type="string" length="255" not-null="false"
	*/
	public String getClientCertificatePassword()
	{
		return clientPassword;
	}
	public void setClientCertificatePassword(String clientPassword)
	{
		this.clientPassword = clientPassword;
	}
	
	private Set<FileData> files;
	/**
		This is the inverse end.
		@see FileData#getFileServer()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`fileserver_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.FileData"
	*/
	Set<FileData> getFiles()
	{
		return files;
	}
	void setFiles(Set<FileData> files)
	{
		this.files = files;
	}
	
}
