/*
	$Id: ArrayIterator.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
	This class implements the {@link Iterator} interface
	for an array of objects.

	@author Nicklas
	@version 2.0
*/
public class ArrayIterator<E>
	implements Iterator<E>
{
	/**
		The array to iterate.
	*/
	private final E[] array;

	/**
		The index of the next element to return.
	*/
	private int index;

	/**
		Create a new <code>ArrayIterator</code> object.
	 	@param array Array to set. 
	*/
	public ArrayIterator(E[] array)
	{
		this.array = array;
		index = 0;
	}
	public boolean hasNext()
	{
		return (array != null) && (index < array.length);
	}
	public E next()
	{
		if (!hasNext()) throw new NoSuchElementException();
		return array[index++];
	}
	/**
		This operation is not supported.
		@throws UnsupportedOperationException Is always thrown
	*/
	public void remove()
	{
		throw new UnsupportedOperationException();
	}
}
