/**
	$Id: ThrowableUtil.java 5114 2009-10-02 12:29:28Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.error;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.PrintWriter;
import java.io.StringWriter;

import net.sf.basedb.core.StringUtil;

/**
	Utility class for working with {@link Throwable}:s.
	
	@author Nicklas
	@version 2.9
	@base.modified $Date: 2009-10-02 14:29:28 +0200 (Fri, 02 Oct 2009) $
*/
public class ThrowableUtil
{

	/**
		Convert a stacktrace to a string that is usually printed
		with the {@link Throwable#printStackTrace()} method.
		@param t The throwable
		@return The string representation of the stack trace
	*/
	public static String stackTraceToString(Throwable t)
	{
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		t.printStackTrace(pw);
		pw.close();
		return sw.toString();
	}
	
	/**
		Prints as much of a stacktrace as possible to an image. 
		@param t The throwable
		@param width The width of the image
		@param height The height of the image
		@since 2.14
	*/
	public static BufferedImage stackTraceToImage(Throwable t, int width, int height)
	{
		BufferedImage img = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D graphics = img.createGraphics();
		graphics.setBackground(Color.WHITE);
		graphics.clearRect(0, 0, width, height);
		graphics.setColor(Color.RED);
		FontRenderContext context = graphics.getFontRenderContext();

		Font topFont = new Font("SansSerif", Font.BOLD, 12);
		Font stackTraceFont = new Font("SansSerif", Font.PLAIN, 12);

		float x = .0f;
		float y = .0f;
		String causedBy = "";
		do
		{
			graphics.setFont(topFont);
			String msg = causedBy + t.getClass().getSimpleName() + ": " + t.getMessage();
			x = 5.0f;
			Rectangle2D bounds = topFont.getStringBounds(msg, context);
			y += bounds.getHeight()+5;
			if (bounds.getWidth() > width - x)
			{
				msg = StringUtil.trimStringMiddle(msg,(int)(msg.length() * (width - x) / bounds.getWidth()));
			}
			graphics.drawString(msg, x, y);
			
			graphics.setFont(stackTraceFont);
			x = 15.0f;
			for (StackTraceElement st : t.getStackTrace())
			{
				msg = "at " + st.getClassName() + "." + st.getMethodName() + 
					":" + st.getLineNumber();
				bounds = stackTraceFont.getStringBounds(msg, context);
				y += bounds.getHeight();
				if (bounds.getWidth() > width - x)
				{
					msg = StringUtil.trimStringMiddle(msg,(int)(msg.length() * (width - x) / bounds.getWidth()));
				}
				graphics.drawString(msg, x, y);
			}
			t = t.getCause();
			causedBy = "Caused by: ";
		} while (t != null && y < height);
		return img;
	}
	
}
