/**
	$Id:ExtensionPointBean.java 4187 2008-03-20 11:15:25Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

/**
	A bean-like implementation of the {@link ExtensionPoint}
	interface. This implementation adds setter methods for simple
	initialisation of the bean properties.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class ExtensionPointBean<A extends Action>
	implements ExtensionPoint<A>
{

	private String id;
	private String name;
	private String description;
	private Class<A> actionClass;
	private RendererFactory<? super A> rendererFactory;
	private boolean allowRendererOverride;
	
	/**
		Create a new empty extension point bean. Use the setter
		methods to set the properties before registering 
		the extension with {@link Registry#registerExtensionPoint(ExtensionPoint)}.
	*/
	public ExtensionPointBean()
	{}
	
	/**
		Create a new fully initialised extension point bean. See the 
		{@link ExtensionPoint} interface for a description of the 
		parameters.
	*/
	public ExtensionPointBean(
		String id, 
		String name,
		String description, 
		Class<A> actionClass, 
		RendererFactory<A> rendererFactory, 
		boolean allowRendererOverride)
	{
		this.id = id;
		this.name = name;
		this.description = description;
		this.actionClass = actionClass;
		this.rendererFactory = rendererFactory;
		this.allowRendererOverride = allowRendererOverride;
	}
	
	@Override
	public String toString()
	{
		return "ExtensionPointBean[id="+ id +"]";
	}
	
	@Override
	public String getId()
	{
		return id;
	}
	public void setId(String id)
	{
		this.id = id;
	}
	
	@Override
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	
	@Override
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	
	@Override
	public Class<A> getActionClass()
	{
		return actionClass;
	}
	public void setActionClass(Class<A> actionClass)
	{
		this.actionClass = actionClass;
	}

	@Override
	public RendererFactory<? super A> getRendererFactory()
	{
		return rendererFactory;
	}
	public void setRendererFactory(RendererFactory<? super A> rendererFactory)
	{
		this.rendererFactory = rendererFactory;
	}
	
	@Override
	public boolean allowRendererOverride()
	{
		return allowRendererOverride;
	}
	public void setAllowRendererOverrider(boolean allowRendererOverride)
	{
		this.allowRendererOverride = allowRendererOverride;
	}


}
