/**
	$Id: PlatformLoader.java 4740 2009-02-05 13:39:25Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PlatformVariant;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.node.NameableNameGenerator;
import net.sf.basedb.util.overview.node.NodeFactory;
import net.sf.basedb.util.overview.node.NodeNameGenerator;

/**
	Node loader implementation for platform and platform variant. Both are 
	loaded as end-point property nodes with no children. It can't be a root 
	node or a forward/reverse-loading node.
	
	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-02-05 14:39:25 +0100 (Thu, 05 Feb 2009) $
*/
public class PlatformLoader
	extends BasicItemNodeLoader<Platform>
{
	
	public PlatformLoader()
	{
		super(Item.PLATFORM, DENY_ROOT_NODE, 
				new NameableNameGenerator<Platform>("platform", "Platform"));
	}
	
	/*
		From the NodeLoader interface
		------------------------------
	*/
	@Override
	public Node createPropertyNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<Platform> nf = getNodeFactory(dc, context);
		Platform platform = null;
		PlatformVariant variant = null;
		boolean denied = false;
		try
		{
			BasicItem parentItem = parentNode.getItem(dc);
			if (parentItem instanceof RawBioAssay)
			{
				RawBioAssay rba = (RawBioAssay)parentItem;
				platform = rba.getPlatform();
				variant = rba.getVariant();
			}
			else if (parentItem instanceof ArrayDesign)
			{
				ArrayDesign ad = (ArrayDesign)parentItem;
				platform = ad.getPlatform();
				variant = ad.getVariant();
			}
			// If there are other items with a platform
			// we should add more cases here
		}
		catch (PermissionDeniedException ex)
		{
			denied = true;
		}
		Node platformNode = createItemNode(nf, platform, platform, denied, parentNode, ChildNodeDirection.NONE);
		if (platformNode != null && variant != null)
		{
			NodeNameGenerator<PlatformVariant> nameGenerator = 
				new NameableNameGenerator<PlatformVariant>("variant", "Variant");
			NodeFactory<PlatformVariant> variantNodeFactory = 
				new NodeFactory<PlatformVariant>(dc, context, null, nameGenerator);
			variantNodeFactory.createNode(variant, variant, platformNode, ChildNodeDirection.NONE);
		}
		return platformNode;
	}
	// -----------------------------------
	
}
