/*
	$Id: NullPlugin.java 5437 2010-10-01 11:51:57Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg
 
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 	You should have received a copy of the GNU General Public License
 	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.DateUtil;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.PathParameterType;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.IntegerParameterType;
import net.sf.basedb.core.LongParameterType;
import net.sf.basedb.core.FloatParameterType;
import net.sf.basedb.core.DoubleParameterType;
import net.sf.basedb.core.DateParameterType;
import net.sf.basedb.core.BooleanParameterType;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.FileParameterType;
import net.sf.basedb.core.MimeType;
import net.sf.basedb.core.File;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.TimestampParameterType;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.plugin.Plugin;
import net.sf.basedb.core.plugin.ParameterValues;
import net.sf.basedb.core.plugin.InteractivePlugin;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.AbstractPlugin;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;

import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.Set;
import java.util.Date;
import java.util.Collections;

/**
 	Just a simple test plugin
 	
	@base.modified $Date$
	@author Enell, Nicklas
	@version 2.0
*/
public class NullPlugin 
	extends AbstractPlugin
	implements InteractivePlugin, NullPluginInterface
{
	private static final Set<GuiContext> guiContexts = 
		Collections.singleton(new GuiContext(Item.REPORTER, GuiContext.Type.LIST));

	private static final PluginParameter<String> stringSection = new PluginParameter<String>(
		"stringSection", "String values", "This section have only string values", null
	);

	private static final PluginParameter<String> oneString = new PluginParameter<String>(
		"oneString", "One string", "Please enter single string value", 
		new StringParameterType(255, null, true, 1, 0, 0)
	);
	private static final PluginParameter<String> manyStrings = new PluginParameter<String>(
		"manyStrings", "Many strings", "Please enter several string values", 
		new StringParameterType(255, null, true, 0, 0, 0)
	);
	private static final PluginParameter<String> selectOneString =  new PluginParameter<String>(
		"selectOneString", "Select one string", "Please select a single string value", 
		new StringParameterType(255, null, true, 1, 0, 0, Arrays.asList("A", "B", "C"))
	);
	private static final PluginParameter<String> selectMultipleStrings =  new PluginParameter<String>(
		"selectMultipleStrings", "Select multiple strings", "Please select any number of string values", 
		new StringParameterType(255, null, true, 0, 0, 0, Arrays.asList("One", "Two", "Three", "Four"))
	);
	
	private static final PluginParameter<String> oneText = new PluginParameter<String>(
			"oneText", "One text", "Please enter single text value", 
			new StringParameterType(null, null, false, 1, 0, 0)
		);
	
	private static final PluginParameter<Integer> integerSection = new PluginParameter<Integer>(
		"integerSection", "Integer values", "This section have only integer values", null
	);

	private static final PluginParameter<Integer> oneInteger = new PluginParameter<Integer>(
		"oneInteger", "One integer", "Please enter single integer value", new IntegerParameterType()
	);
	private static final PluginParameter<Integer> manyIntegers = new PluginParameter<Integer>(
		"manyIntegers", "Many integers", "Please enter several integer values", 
		new IntegerParameterType(1, 100, null, false, 0, 0, 0, null)
	);
	private static final PluginParameter<Integer> selectOneInteger =  new PluginParameter<Integer>(
		"selectOneInteger", "Select one integer", "Please select a single integer value", 
		new IntegerParameterType(null, null, null, false, 1, 0, 0, Arrays.asList(1, 2, 3))
	);
	private static final PluginParameter<Integer> selectMultipleIntegers =  new PluginParameter<Integer>(
		"selectMultipleIntegers", "Select multiple integers", "Please select any number of integer values", 
		new IntegerParameterType(null, null, null, false, 0, 0, 0, Arrays.asList(10, 20, 30, 40))
	);
	
	private static final PluginParameter<Long> longSection = new PluginParameter<Long>(
		"longSection", "Long values", "This section have only long values", null
	);

	private static final PluginParameter<Long> oneLong = new PluginParameter<Long>(
		"oneLong", "One long", "Please enter single long value", new LongParameterType()
	);
	private static final PluginParameter<Long> manyLongs = new PluginParameter<Long>(
		"manyLongs", "Many longs", "Please enter several long values", 
		new LongParameterType(1L, 100L, null, false, 0, 0, 0, null)
	);
	private static final PluginParameter<Long> selectOneLong =  new PluginParameter<Long>(
		"selectOneLong", "Select one long", "Please select a single long value", 
		new LongParameterType(null, null, null, false, 1, 0, 0, Arrays.asList(1L, 2L, 3L))
	);
	private static final PluginParameter<Long> selectMultipleLongs =  new PluginParameter<Long>(
		"selectMultipleLongs", "Select multiple longs", "Please select any number of long values", 
		new LongParameterType(null, null, null, false, 0, 0, 0, Arrays.asList(10L, 20L, 30L, 40L))
	);
	
	private static final PluginParameter<Float> floatSection = new PluginParameter<Float>(
		"floatSection", "Float values", "This section have only float values", null
	);
	private static final PluginParameter<Float> oneFloat = new PluginParameter<Float>(
		"oneFloat", "One float", "Please enter single float value", new FloatParameterType()
	);
	private static final PluginParameter<Float> manyFloats = new PluginParameter<Float>(
		"manyFloats", "Many floats", "Please enter several float values", 
		new FloatParameterType(1.0f, 100.0f, null, false, 0, 0, 0, null)
	);
	private static final PluginParameter<Float> selectOneFloat =  new PluginParameter<Float>(
		"selectOneFloat", "Select one float", "Please select a single float value", 
		new FloatParameterType(null, null, null, false, 1, 0, 0, Arrays.asList(1.1f, 2.2f, 3.3f))
	);
	private static final PluginParameter<Float> selectMultipleFloats =  new PluginParameter<Float>(
		"selectMultipleFloats", "Select multiple floats", "Please select any number of float values", 
		new FloatParameterType(null, null, null, false, 0, 0, 0, Arrays.asList(11.1f, 22.2f, 33.3f, 44.4f))
	);
	
	private static final PluginParameter<Double> doubleSection = new PluginParameter<Double>(
		"doubleSection", "Double values", "This section have only double values", null
	);
	private static final PluginParameter<Double> oneDouble = new PluginParameter<Double>(
		"oneDouble", "One double", "Please enter single double value", new DoubleParameterType()
	);
	private static final PluginParameter<Double> manyDoubles = new PluginParameter<Double>(
		"manyDoubles", "Many doubles", "Please enter several double values", 
		new DoubleParameterType(1.0, 100.0, null, false, 0, 0, 0, null)
	);
	private static final PluginParameter<Double> selectOneDouble =  new PluginParameter<Double>(
		"selectOneDouble", "Select one double", "Please select a single double value", 
		new DoubleParameterType(null, null, null, false, 1, 0, 0, Arrays.asList(1.1, 2.2, 3.3))
	);
	private static final PluginParameter<Double> selectMultipleDoubles =  new PluginParameter<Double>(
		"selectMultipleDoubles", "Select multiple doubles", "Please select any number of double values", 
		new DoubleParameterType(null, null, null, false, 0, 0, 0, Arrays.asList(11.1, 22.2, 33.3, 44.4))
	);

	private static final PluginParameter<Date> dateSection = new PluginParameter<Date>(
		"dateSection", "Date values", "This section have only date values", null
	);

	private static final List<Date> dates = Arrays.asList(
		DateUtil.parseString("2005-11-24"),
		DateUtil.parseString("2006-11-24"),
		DateUtil.parseString("2007-11-24")
	);
	private static final PluginParameter<Date> oneDate = new PluginParameter<Date>(
		"oneDate", "One date", "Please enter single date value", new DateParameterType()
	);
	private static final PluginParameter<Date> manyDates = new PluginParameter<Date>(
		"manyDates", "Many dates", "Please enter several date values", 
		new DateParameterType(null, false, 0, 0, 0, null)
	);
	private static final PluginParameter<Date> selectOneDate =  new PluginParameter<Date>(
		"selectOneDate", "Select one date", "Please select a single date value", 
		new DateParameterType(null, false, 1, 0, 0, dates)
	);
	private static final PluginParameter<Date> selectMultipleDates =  new PluginParameter<Date>(
		"selectMultipleDates", "Select multiple dates", "Please select any number of date values", 
		new DateParameterType(null, false, 0, 0, 0, dates)
	);

	private static final PluginParameter<Date> timestampSection = new PluginParameter<Date>(
			"timestampSection", "Timestamp values", "This section have only timestamp values", null
		);
	private static final List<Date> timestamps = Arrays.asList(
		DateUtil.parseTimestamp("2005-11-24 12:12:12"),
		DateUtil.parseTimestamp("2006-11-24 13:13:13"),
		DateUtil.parseTimestamp("2007-11-24 14:14:14")
	);
	private static final PluginParameter<Date> oneTimestamp = new PluginParameter<Date>(
		"oneTimestamp", "One timestamp", "Please enter single timestamp value", new TimestampParameterType()
	);
	private static final PluginParameter<Date> manyTimestamps = new PluginParameter<Date>(
		"manyTimestamps", "Many timestamps", "Please enter several timestamp values", 
		new TimestampParameterType(null, false, 0, 0, 0, null)
	);
	private static final PluginParameter<Date> selectOneTimestamp =  new PluginParameter<Date>(
		"selectOneTimestamp", "Select one timestamp", "Please select a single timestamp value", 
		new TimestampParameterType(null, false, 1, 0, 0, timestamps)
	);
	private static final PluginParameter<Date> selectMultipleTimestamps =  new PluginParameter<Date>(
		"selectMultipleTimestamps", "Select multiple timestamps", "Please select any number of timestamp values", 
		new TimestampParameterType(null, false, 0, 0, 0, timestamps)
	);
	
	private static final PluginParameter<Boolean> booleanSection = new PluginParameter<Boolean>(
		"booleanSection", "Boolean values", "This section have only boolean values", null
	);
	private static final PluginParameter<Boolean> oneBoolean = new PluginParameter<Boolean>(
		"oneBoolean", "One boolean", "Please enter single boolean value", new BooleanParameterType()
	);

	private static final PluginParameter<String> itemSection = new PluginParameter<String>(
		"itemSection", "Item values", "This section have only item values", null
	);

	private static final PluginParameter<File> oneFile = new PluginParameter<File>(
		"oneFile", "One file", "Please enter single file value", new FileParameterType()
	);
	private static final PluginParameter<File> manyFiles = new PluginParameter<File>(
		"manyFiles", "Many files", "Please enter several file values", 
		new FileParameterType(null, false, 0)
	);
	
	private static final PluginParameter<String> oneDirectory = new PluginParameter<String>(
		"oneDirectory", "Save to directory", "Please specify the output directory where the plugin should store it's generated files",
		new PathParameterType(Path.Type.DIRECTORY)
	);
	
	private static final PluginParameter<String> onePath = new PluginParameter<String>(
			"onePath", "Save as", "Please specify the file name where the plugin should store it's output",
			new PathParameterType(Path.Type.FILE)
		);
	
	private static final PluginParameter<String> hidden = new PluginParameter<String>(
			"hidden", null, null, new StringParameterType(255, "A hidden string", true)
		);

	private RequestInformation configurePlugin;
	private RequestInformation configureJob;

	/*
		From the Plugin interface
		-------------------------------------------
	*/
	public Plugin.MainType getMainType()
	{
		return Plugin.MainType.OTHER;
	}

	public About getAbout()
	{
		return new AboutImpl
		(
			"Null plugin",
			"This plugin does nothing. It is only used for testing purposes.",
			"2.0",
			"2006, Base 2 development team",
			null,
			null,
			"http://base.thep.lu.se"
		);
	}
	public boolean supportsConfigurations()
	{
		return true;
	}
	public boolean requiresConfiguration()
	{
		return false;
	}
	
	public void init(SessionControl sc, ParameterValues configuration, ParameterValues job)
		throws BaseException
	{
		super.init(sc, configuration, job);
		ArrayList<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>(40);
		parameters.add(stringSection);
		parameters.add(oneString);
		parameters.add(manyStrings);
		parameters.add(selectOneString);
		parameters.add(selectMultipleStrings);
		parameters.add(oneText);
		
		parameters.add(integerSection);
		parameters.add(oneInteger);
		parameters.add(manyIntegers);
		parameters.add(selectOneInteger);
		parameters.add(selectMultipleIntegers);
		
		parameters.add(longSection);
		parameters.add(oneLong);
		parameters.add(manyLongs);
		parameters.add(selectOneLong);
		parameters.add(selectMultipleLongs);
		
		parameters.add(floatSection);
		parameters.add(oneFloat);
		parameters.add(manyFloats);
		parameters.add(selectOneFloat);
		parameters.add(selectMultipleFloats);
		
		parameters.add(doubleSection);
		parameters.add(oneDouble);
		parameters.add(manyDoubles);
		parameters.add(selectOneDouble);
		parameters.add(selectMultipleDoubles);
		
		parameters.add(dateSection);
		parameters.add(oneDate);
		parameters.add(manyDates);
		parameters.add(selectOneDate);
		parameters.add(selectMultipleDates);

		parameters.add(timestampSection);
		parameters.add(oneTimestamp);
		parameters.add(manyTimestamps);
		parameters.add(selectOneTimestamp);
		parameters.add(selectMultipleTimestamps);
		
		parameters.add(booleanSection);
		parameters.add(oneBoolean);
		
		parameters.add(itemSection);
		
		DbControl dc = sc.newDbControl();
		ItemQuery<MimeType> query = MimeType.getQuery();
		query.order(Orders.asc(Hql.property("name")));
		List<MimeType> mimeTypes = new ArrayList<MimeType>(query.list(dc));
		dc.close();
		
		PluginParameter<MimeType> selectOneMimeTypes = new PluginParameter<MimeType>(
			"oneMimeType", "Select one MIME type", "Please select a single MIME type value", 
			new ItemParameterType<MimeType>(MimeType.class, null, false, 1, mimeTypes)
		);
		PluginParameter<MimeType> selectMultipleMimeTypes = new PluginParameter<MimeType>(
			"multipleMimeTypes", "Select multiple MIME types", "Please select any number of MIME type values", 
			new ItemParameterType<MimeType>(MimeType.class, null, false, 0, mimeTypes)
		);
		parameters.add(selectOneMimeTypes);
		parameters.add(selectMultipleMimeTypes);
		parameters.add(oneFile);
		parameters.add(manyFiles);
		parameters.add(oneDirectory);
		parameters.add(onePath);
		parameters.add(hidden);

		configurePlugin = new RequestInformation(
			Request.COMMAND_CONFIGURE_PLUGIN,
			"Configuration settings",
			"Please enter values for the following parameters to configure the plugin",
			parameters
		);
		
		configureJob = new RequestInformation(
			Request.COMMAND_CONFIGURE_JOB,
			"Job settings",
			"Please enter values for the following parameters to configure the job",
			parameters,
			"test_nullplugin.jsp"
		);
	}
	
	/**
		Just print some info to the ProgressReporter for the Request COMMAND_EXECUTE.
	*/
	public void run(Request request, Response response, ProgressReporter progress)
	{
		String command = request.getCommand();
		if (command.equals(Request.COMMAND_EXECUTE))
		{
			progress.display(0, "starting");
			try
			{
				for (int i = 1; i <= 4; i++)
				{
					progress.display(100*i/4, "processing");
					progress.append("...");
				}
				progress.append("done\n");
				response.setDone("Ok");
			}
			catch (BaseException e)
			{
				response.setError("NullPlugin.invoke()#Request.COMMAND_EXECUTE", Arrays.asList(e));
			}
		}
	}
	// -------------------------------------------
	/*
		From the InteractivePlugin interface
		-------------------------------------------
	*/
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	public String isInContext(GuiContext context, Object item)
	{
		return null;
	}
	/**
		Returns parameter configPlugin or configJob for Request COMMAND_CONFIGURE_PLUGIN 
		respectively COMMAND_CONFIGURE_JOB.
	*/
	public RequestInformation getRequestInformation(GuiContext context, String command)
	{
		RequestInformation requestInformation = null;
		if (command.equals(Request.COMMAND_CONFIGURE_PLUGIN))
		{
			requestInformation = configurePlugin;
		}
		else if (command.equals(Request.COMMAND_CONFIGURE_JOB))
		{
			requestInformation = configureJob;
		}
		return requestInformation;
	}

	public void configure(GuiContext context, Request request, Response response)
	{
		String command = request.getCommand();
		
		List<PluginParameter<?>> parameters = null;
		ParameterValues values = null;
		
		if (command.equals(Request.COMMAND_CONFIGURE_PLUGIN))
		{
			parameters = configurePlugin.getParameters();
			values = configuration;
		}
		else if (command.equals(Request.COMMAND_CONFIGURE_JOB))
		{
			parameters = configureJob.getParameters();
			values = job;
		}
		
		List<Throwable> errors = validateRequestParameters(parameters, request);
		if (errors != null)
		{
			response.setError(errors.size()+" invalid parameter(s) were found in the request", errors);
			return;
		}
			
		for (PluginParameter<?> pp : parameters)
		{
			if (pp.getParameterType() != null)
			{
				storeValues(values, request, pp);
			}
		}
		response.setDone("Plugin configuration complete");
	}
	
	// -------------------------------------------
	/*
		From the NullPluginInterface interface
		-------------------------------------------
	*/
	public void downloadFromRepository(String repositoryPath)
	{}
	// -------------------------------------------


}
