/*
	$Id: TestArrayDesign.java 5340 2010-05-10 06:15:00Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.ArrayDesignBlock;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BlockInfo;
import net.sf.basedb.core.DataQuery;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Feature;
import net.sf.basedb.core.FeatureBatcher;
import net.sf.basedb.core.FeatureIdentificationMethod;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemAlreadyExistsException;
import net.sf.basedb.core.ItemProxy;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.DataResultIterator;
import net.sf.basedb.core.Plate;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.Reporter;
import net.sf.basedb.core.ReporterBatcher;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Trashcan;
import net.sf.basedb.core.Well;
import net.sf.basedb.core.data.FeatureData;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.util.FileUtil;
import net.sf.basedb.util.parser.FlatFileParser;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

public class TestArrayDesign
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing array design");
		write_header();
		// Standard tests: create, load, list
		int id = test_create(Platform.GENERIC, true);
		int id2 = test_create(Platform.GENERIC, false);
		int id3 = test_create(Platform.AFFYMETRIX, true);
		int id4 = test_create(Platform.AFFYMETRIX, false);
		int id6 = test_create(Platform.GENERIC, false);
		test_load(id);
		test_list(-1);

		TestReporter.test_import_from_file("data/test.reportermap.import.txt", 
			"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"ID\"\\t\"Name\".*", "\\t", 3, 4);
		test_import_from_file(id2, "data/test.reportermap.import.txt", 
			"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"ID\"\\t\"Name\".*", null, 0, 2, 1, 3);
		test_add_features(id, FeatureIdentificationMethod.COORDINATES, 10);
		test_add_features(id6, FeatureIdentificationMethod.FEATURE_ID, 10);

		write_feature_header();
		test_list_features(id, -1);
		test_list_features(id3, -1);
		
		// Create arraydesign with wells
		TestReporter.test_import_from_file("data/test.plate96.import.txt", 
			"Plate\\tRow\\tColumn\\tCluster ID\\tClone ID\\tGene Symbol\\tBarcode", "\\t", 4, 5);
		int plateGeometry = TestPlateGeometry.test_create(8, 12, false);
		int plateType = TestPlateType.test_create(plateGeometry, false);
		int[] plateIds = TestPlate.test_import_from_file("data/test.plate96.import.txt", 
			"Plate\\tRow\\tColumn\\tCluster ID\\tClone ID\\tGene Symbol\\tBarcode", plateType, 0, 1, 2, 4);
		int id5 = test_create(Platform.GENERIC, false);
		test_add_plates(id5, plateIds);
		
		write_plate_header();
		test_list_plates(id5, plateIds.length);
		
		test_add_features_from_wells(id5, plateIds, 10);
		
		// Array designs with data in files
		int cdfId = TestFile.test_create("data/test.affymetrix.cdf", false, false);
		test_set_file(id3, DataFileType.AFFYMETRIX_CDF, cdfId);
		test_list_files(id3, 1);

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Standard test: Delete
		test_delete(id);
		test_delete(id2);
		test_delete(id3);
		test_delete(id4);
		test_delete(id5);
		test_delete(id6);
		
		TestPlate.test_delete_all();
		TestPlateType.test_delete(plateType);
		TestPlateGeometry.test_delete(plateGeometry);
		TestFile.test_delete(cdfId);
		TestReporter.test_delete();

		write("++Testing array design "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String platformId, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.ARRAYDESIGN)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Platform platform = Platform.getByExternalId(dc, platformId);
			ArrayDesign ad = ArrayDesign.getNew(dc, platform);
			if (setAll)
			{
				ad.setName("Test array design");
				ad.setDescription("Added at "+new Date());
			}
			dc.saveItem(ad);
			dc.commit();
			id = ad.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(ad, false);
			write_item(0, ad);
			write("--Create array design OK");
		}
		catch (Throwable ex)
		{
			write("--Create array design FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, id);
			write_item(0, ad);
			write("--Load array design OK");
		}
		catch (Throwable ex)
		{
			write("--Load array design FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemQuery<ArrayDesign> q = ArrayDesign.getQuery();
			ItemResultList<ArrayDesign> l = q.list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List array designs OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List array designs FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, id);
			dc.deleteItem(ad);
			Set<ItemProxy> using = ad.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(ad + " is used by " + using);
			}
			dc.commit();
			write("--Delete array design OK");
		}
		catch (Throwable ex)
		{
			write("--Delete array design FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete_all()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			List<ArrayDesign> l = new ArrayList<ArrayDesign>(ArrayDesign.getQuery().list(dc));
			dc.commit();
			Trashcan.delete(TestUtil.getSessionControl(), l, true, null);
			write("--Delete all array designs OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--Delete all array designs FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tAnnotated\tAffyChipDesign");
			write("-- \t-- \t--------- \t-----------\t---------\t--------------");
		}
	}
	
	static void write_feature_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tBlock\tRow\tColumn\tPosition\tReporter\tWell");
			write("-- \t-- \t-----\t---\t------\t--------\t--------\t----");
		}
	}

	static void write_plate_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tPlate    \tPosition");
			write("-- \t-- \t---------\t-----");
		}
	}
	
	static void write_item(int i, ArrayDesign ad)
		throws BaseException
	{
		if (!TestUtil.getSilent())
		{
			write(i+":\t"+ad.getId()+"\t"+ad.getName()+"\t"+ad.getDescription()+"\t"+ad.isAnnotated()+"\t"+ad.getPlatform());
		}
	}

	static void write_item(int i, FeatureData f, DbControl dc)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			write(i+":\t"+f.getId()+"\t"+Feature.getArrayDesignBlock(dc, f)+"\t"+f.getRow()+"\t"+f.getColumn()+"\t"+f.getPosition()+"\t"+f.getReporter()+"\t"+Feature.getWell(dc, f));
		}
	}
	
	static void write_item(int i, Plate p, int index)
		throws BaseException
	{
		if (!TestUtil.getSilent())
		{
			write(i + ":\t" + p.getId() + "\t" + p.getName() + "\t"+index);
		}
	}

	static void write_item(int i, FileSetMember member)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+member.getId()+"\t"+member.getFile().getName()+"\t"+
				member.getDataFileType().getName());
		}
	}	
	
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_add_features(int arrayDesignId, FeatureIdentificationMethod fiMethod, int numFeatures)
	{
		if (arrayDesignId == 0) return;
		DbControl dc = null;
		long time;
		try
		{
			write("--Start adding features ("+numFeatures+" features)");
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			
			ArrayList<Integer> reporterIds = new ArrayList<Integer>();
			DataResultIterator<ReporterData> it = Reporter.getQuery().iterate(dc);
			while(it.hasNext())
			{
				ReporterData reporter = it.next();
				reporterIds.add(reporter.getId());
			}
			
			FeatureBatcher batcher = ad.getFeatureBatcher(fiMethod, null);
			int block = 1;
			int row = 1;
			int column = 1;
			int position = 1;
			ArrayDesignBlock adBlock = ad.addArrayDesignBlock(new BlockInfo(block, (int)Math.ceil(block/4F), ((block-1)%4)+1));
			time = System.currentTimeMillis();
			for (int i = 0; i < numFeatures; i++)
			{
				ReporterData reporter = null;
				if(reporterIds.size() > 0)
				{
					reporter = Reporter.getById(dc, reporterIds.get((int) Math.floor(reporterIds.size() * Math.random())));
				}
				FeatureData data = batcher.newFeature(adBlock, reporter);
				data.setRow(row);
				data.setColumn(column);
				data.setPosition(position);
				data.setExternalId(reporter.getExternalId() + "." + position);

				batcher.insert(data);
				row++;
				position++;
				if (row == 11)
				{
					row = 1;
					column++;
					if (column == 11)
					{
						column = 1;
						block++;
						adBlock = ad.addArrayDesignBlock(new BlockInfo(block, (int)Math.ceil(block/4F), ((block-1)%4)+1));
					}
				}
			}
			dc.commit();
			time = System.currentTimeMillis()-time;
			write("--Add features OK ("+time+" ms)");
		}
		catch (Throwable ex)
		{
			write("--Add features FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	private static void test_add_features_from_wells(int arrayDesignId, int[] plateIds, int numFeatures)
	{
		if (arrayDesignId == 0) return;
		DbControl dc = null;
		long time;
		try
		{
			write("--Start adding features from wells ("+numFeatures+" features)");
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			if (Platform.AFFYMETRIX.equals(ad.getPlatform().getExternalId()))
			{
				throw new BaseException("Cannot add features from wells on an affy arraydesign");
			}
			ArrayList<Well> wells = new ArrayList<Well>();
			for (int plateId : plateIds)
			{
				Plate plate = Plate.getById(dc, plateId);
				wells.addAll(plate.getWells().list(dc));
			}
			FeatureBatcher batcher = ad.getFeatureBatcher(FeatureIdentificationMethod.COORDINATES, null);
			int block = 1;
			int row = 1;
			int column = 1;
			ArrayDesignBlock adBlock = ad.addArrayDesignBlock(new BlockInfo(block, (int)Math.ceil(block/4F), ((block-1)%4)+1));
			time = System.currentTimeMillis();
			for (int i = 0; i < numFeatures; i++)
			{
				Well well = null;
				if (!wells.isEmpty())
				{
					well = wells.get((int) Math.floor(wells.size() * Math.random()));
				}
				FeatureData data = (FeatureData) batcher.newFeature(adBlock, well);
				data.setRow(row);
				data.setColumn(column);

				batcher.insert(data);
				row++;
				if (row == 3)
				{
					row = 1;
					column++;
					if (column == 3)
					{
						column = 1;
						block++;
						adBlock = ad.addArrayDesignBlock(new BlockInfo(block, (int)Math.ceil(block/4F), ((block-1)%4)+1));
					}
				}
			}
			dc.commit();
			time = System.currentTimeMillis()-time;
			write("--Add features from wells OK ("+time+" ms)");
		}
		catch (Throwable ex)
		{
			write("--Add features from wells FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_add_plates(int arrayDesignId, int[] plateIds)
	{
		if (!TestUtil.hasPermission(Permission.WRITE, Item.ARRAYDESIGN) || arrayDesignId == 0 || plateIds == null) return ;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			for (int plateId : plateIds)
			{
				Plate p = Plate.getById(dc, plateId);
				ad.addPlate(p);
			}
			dc.commit();
			write("--Add plates to array design OK");
		}
		catch (Throwable ex)
		{
			write("--Add plates to array design FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_features(int arrayDesignId, int expectedResults)
	{
		if (arrayDesignId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			
			DataQuery<FeatureData> query = ad.getFeatures();
			query.join(Hql.innerJoin(null, "arrayDesignBlock", "blk", null, true));
			query.join(Hql.leftJoin(null, "reporter", "rpt", null, true));
			DataResultIterator<FeatureData> it = query.iterate(dc);
			int counter = 0;
			while (it.hasNext())
			{
				write_item(counter, it.next(), dc);
				++counter;
			}
			if (expectedResults >= 0 && expectedResults != counter)
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+counter);
			}
			write("--List features OK ("+counter+")");
		}
		catch (Throwable ex)
		{
			write("--List features FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_plates(int arrayDesignId, int expectedResults)
	{
		if (arrayDesignId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			
			ItemQuery<Plate> plates = ad.getPlates();
			int counter = 0;
			for (Plate p : plates.list(dc))
			{
				write_item(counter, p, ad.getPlatePosition(p));
				counter++;
			}
			if (expectedResults >= 0 && expectedResults != counter)
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+counter);
			}
			write("--List plates OK ("+counter+")");
		}
		catch (Throwable ex)
		{
			write("--List plates FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_import_from_file(int arrayDesignId, String filename, String headerRegexp, 
			String ignoreRegexp, int blockCol, int rowCol, int colCol, int reporterCol)
	{
		if (arrayDesignId == 0) return;
		write("--Start import features from file (" + filename + ")");
		DbControl dc = null;
		long time;
		int i = 0;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			FeatureBatcher featureBatcher = ad.getFeatureBatcher(FeatureIdentificationMethod.COORDINATES, null);
			ReporterBatcher reporterBatcher = ReporterBatcher.getNew(dc);
		
			FlatFileParser parser = new FlatFileParser();
			parser.setDataHeaderRegexp(Pattern.compile(headerRegexp));
			parser.setDataSplitterRegexp(Pattern.compile("\\t"));
			if (ignoreRegexp != null)
			{
				parser.setIgnoreRegexp(Pattern.compile(ignoreRegexp));
			}
			parser.setInputStream(FileUtil.getInputStream(new java.io.File(filename)), "ISO-8859-1");
			parser.parseHeaders();
			
			ArrayDesignBlock adBlock = null;
			
			time = System.currentTimeMillis();
			while (parser.hasMoreData())
			{
				FlatFileParser.Data parsedData = parser.nextData();
				String reporterId = parsedData.get(reporterCol);
				ReporterData reporter = reporterId == null ? null : reporterBatcher.getByExternalId(reporterId);
				int block = Integer.valueOf(parsedData.get(blockCol));
				if (adBlock == null || adBlock.getBlockNumber() != block)
				{
					adBlock = ad.addArrayDesignBlock(new BlockInfo(block, 0, 0));
				}
				FeatureData featureData = featureBatcher.newFeature(adBlock, reporter);
				featureData.setRow(Integer.valueOf(parsedData.get(rowCol)));
				featureData.setColumn(Integer.valueOf(parsedData.get(colCol)));

				try
				{
					featureBatcher.insert(featureData);
					i++;
				}
				catch (ItemAlreadyExistsException ex)
				{
					// Ignore duplicate features
				}
			}
			dc.commit();
			time = System.currentTimeMillis()-time;

			write("--Import features from file OK ("+i+", time="+time+" ms)");
		}
		catch (Throwable ex)
		{
			write("--Import features from file FAILED ("+i+")");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_set_file(int arrayDesignId, String fileType, int fileId)
	{
		if (arrayDesignId == 0 || fileId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			DataFileType type = DataFileType.getByExternalId(dc, fileType);
			File file = File.getById(dc, fileId);
			ad.getFileSet().setMember(file, type);
			ad.getFileSet().validate(dc, true);
			dc.commit();
			write("--Set file OK ("+fileType+")");
		}
		catch (Throwable ex)
		{
			write("--Set file FAILED ("+fileType+")");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
		
	static void test_list_files(int arrayDesignId, int expectedResults)
	{
		if (arrayDesignId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ArrayDesign ad = ArrayDesign.getById(dc, arrayDesignId);
			ItemResultList<FileSetMember> l = ad.getFileSet().getMembers().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List files OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List files FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}

