/*
 $Id$

 Copyright (C) 2006 Johan Enell
 Copyright (C) 2007 Nicklas Nordborg

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import java.util.Arrays;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.plugin.Response;

public class TestBase1PluginExecuter
{
	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		boolean wasSetup = TestAnalyzeUtil.setupExperiment();
		write("++Testing TestBase1PluginExecuter");
		
		try
		{
			int experimentId = TestAnalyzeUtil.getItemId("experiment.genepix");
			int rootBasId = TestAnalyzeUtil.getItemId("bioassayset.root");
			if (experimentId == 0 || rootBasId == 0)
			{
				ok = false;
				write("Test experiment was not setup correctly.");
				return ok;
			}
			
			int cloneId = TestAnalyzeUtil.clone_bioassay_set(rootBasId, "Base1PluginExecuter clone");
	
			int filteredBasId = TestBasicAnalysis.test_filter_bioassayset(cloneId, "Filtered; ch1 > ch2", 1);
	
			// Formulas that are used in configuration files as exported columns
			int posFormulaId = TestFormula.test_create("position",
					Formula.Type.COLUMN_EXPRESSION, "genepix", new String[] { "pos()" }, false);
			int reporterFormulaId = TestFormula.test_create("reporter",
					Formula.Type.COLUMN_EXPRESSION, "genepix", new String[] { "rep('id')" }, false);
			int externalIdFormulaId = TestFormula.test_create("reporterId",
					Formula.Type.COLUMN_EXPRESSION, "genepix", new String[] { "rep('externalId')" }, false);
	
			// Configuration files and configurations
			int fileSerialId = TestFile.test_create("data/test.nullplugin.base", false, false);
			int fileMatrixId = TestFile.test_create("data/test.nullplugin.matrix.base", false, false);
			int fileCleanStdId = TestFile.test_create("data/test.nullplugin.cleanstd.base", false, false);
			int fileMergeId = TestFile.test_create("data/test.base1plugin.base", false, false);
			int directoryId = TestDirectory.test_create(true, "Base1PluginExecuter");
			
			int pluginDefinitionId = TestPluginDefinition.test_get("net.sf.basedb.plugins.Base1PluginExecuter");
			int cloneSerialKeepStdId = test_create_configuration(pluginDefinitionId, fileSerialId);
			int cloneMatrixKeepStd = test_create_configuration(pluginDefinitionId, fileMatrixId);
			int cloneSerialCleanStdId = test_create_configuration(pluginDefinitionId, fileCleanStdId);
			int mergeId = test_create_configuration(pluginDefinitionId, fileMergeId);
			
			// Calculate extra value used by one of the configurations
			int aExtraValueId = TestExtraValueType.test_create("a", Type.FLOAT, false);
			TestBasicAnalysis.test_calculate_spotextravalues(cloneId, null, 0, aExtraValueId, 0);
			TestBasicAnalysis.test_calculate_spotextravalues(filteredBasId, null, 0, aExtraValueId, 0);
			
			// Copy input to output - serial format
			int jobId = test_create_job(cloneSerialKeepStdId, experimentId, cloneId, 
					directoryId, "Clone serial (keep std*.txt)");
			ok &= TestJob.test_execute(jobId, false);
			
			// Copy input to output - serial format; remove stdin and stdout
			int jobIdCleanStd = test_create_job(cloneSerialCleanStdId, experimentId, 
					cloneId, directoryId, "Clone serial (remove std*.txt)");
			ok &= TestJob.test_execute(jobIdCleanStd, false);
			
			// Copy input to output - matrix format
			int jobIdMatrix = test_create_job(cloneMatrixKeepStd, experimentId, 
					cloneId, directoryId, "Clone matrix (keep std*.txt)");
			ok &= TestJob.test_execute(jobIdMatrix, false);
			
			// Copy input to output - matrix format - filtered bioassayset
			int jobIdMatrix2 = test_create_job(cloneMatrixKeepStd, experimentId, 
					filteredBasId, directoryId, "Clone filtered matrix (keep std*.txt)");
			ok &= TestJob.test_execute(jobIdMatrix2, false);
			
			// Merge assays +  create subdir
			int jobId3 = test_create_job(mergeId, experimentId, 
					cloneId, directoryId, "Merge assays + create sudir",
					new TestJob.Parameter("mergeAssays", "1"), 
					new TestJob.Parameter("createSubdir", "1"));
			ok &= TestJob.test_execute(jobId3, false);
	
			// Offset positions
			int jobId4 = test_create_job(mergeId, experimentId, 
					cloneId, directoryId, "Offset positions",
					new TestJob.Parameter("offsetPositions", "1"));
			ok &= TestJob.test_execute(jobId4, false);
	
			// Offset positions and merge assays
			int jobId5 = test_create_job(mergeId, experimentId,
					cloneId, directoryId, "Offset position + merge assays",
					new TestJob.Parameter("mergeAssays", "1"), 
					new TestJob.Parameter("offsetPositions", "1"));
			ok &= TestJob.test_execute(jobId5, false);
	
			// Create reporter list
			int jobId6 = test_create_job(mergeId, experimentId, 
					cloneId, directoryId, "Create reporter list",
					new TestJob.Parameter("reporterList", "1"));
			ok &= TestJob.test_execute(jobId6, false);
			
			if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
			
			// Cleanup
			TestAnalyzeUtil.test_delete_bioassayset(cloneId);
			TestJob.test_delete(jobId);
			TestJob.test_delete(jobId3);
			TestJob.test_delete(jobId4);
			TestJob.test_delete(jobId5);
			TestJob.test_delete(jobId6);
			TestJob.test_delete(jobIdCleanStd);
			TestJob.test_delete(jobIdMatrix);
			TestJob.test_delete(jobIdMatrix2);
					
			TestExtraValueType.test_delete(aExtraValueId);
			TestPluginConfiguration.test_delete(mergeId);
			TestPluginConfiguration.test_delete(cloneSerialKeepStdId);
			TestPluginConfiguration.test_delete(cloneMatrixKeepStd);
			TestPluginConfiguration.test_delete(cloneSerialCleanStdId);
	
			TestFormula.test_delete(posFormulaId);
			TestFormula.test_delete(reporterFormulaId);
			TestFormula.test_delete(externalIdFormulaId);
			TestFile.test_delete(fileSerialId);
			TestFile.test_delete(fileMergeId);
			TestFile.test_delete(fileMatrixId);
			TestFile.test_delete(fileCleanStdId);
			TestDirectory.test_delete(directoryId, true);
		}
		finally
		{
			if (wasSetup) TestAnalyzeUtil.cleanupExperiment();
			write("++Testing TestBase1PluginExecuter "+(ok ? "OK" : "Failed")+"\n");
		}
		return ok;
	}
	
	static int test_create_configuration(int pluginDefinitionId, int fileId)
	{
		if (pluginDefinitionId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition pd = PluginDefinition.getById(dc, pluginDefinitionId);
			PluginConfiguration pc = pd.newPluginConfiguration();
			pc.setName(pd.getName());

			PluginConfigurationRequest request = pc.configure();
			request.setParameterValue("execPath", new java.io.File("").getAbsolutePath()+java.io.File.separator);
			request.setParameterValue("file", File.getById(dc, fileId));

			write_request_information(request.getRequestInformation());
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				
				if (TestUtil.isWindows())
				{
					String executable = (String)pc.getParameterValues("execName").get(0);
					String winExecutable = executable.replace(".sh", ".bat");
					pc.setParameterValues("execName", new StringParameterType(255, null, true), Arrays.asList(winExecutable));
				}
				
				dc.saveItem(pc);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = pc.getId();
			write("--Create configuration for Base1PluginExecuter OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for Base1PluginExecuter FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static int test_create_job(int pluginConfigurationId, int experimentId, int bioAssaySetId, 
			int directoryId, String childName, TestJob.Parameter... parameters)
	{
		if (pluginConfigurationId == 0 || experimentId == 0 || bioAssaySetId == 0 || 
				!TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			Experiment e = Experiment.getById(dc, experimentId);
			BioAssaySet bioAssaySet = BioAssaySet.getById(dc, bioAssaySetId);
			dc.getSessionControl().getCurrentContext(Item.EXPERIMENT).setId(experimentId);
			dc.getSessionControl().getCurrentContext(Item.BIOASSAYSET).setId(bioAssaySetId);
			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(e);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(null);
			
			// First, set default values
			for (PluginParameter<?> p : request.getRequestInformation().getParameters())
			{
				if (p.getParameterType() != null)
				{
					request.setParameterValue(p.getName(), p.getParameterType().getDefaultValue());
				}
			}
			
			// Then, set values from method call
			if (parameters != null)
			{
				for (TestJob.Parameter p : parameters)
				{
					request.setParameterValue(p.name, p.value);
				}
			}
			
			request.setParameterValue("source", bioAssaySet);
			request.setParameterValue("pluginDirectory", Directory.getById(dc, directoryId).getPath().toString());
			request.setParameterValue("childName", childName);
			write_request_information(request.getRequestInformation());
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for Base1PluginExecuter OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for Base1PluginExecuter FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
		
}
