/*
 $Id: TestDataFileType.java 5340 2010-05-10 06:15:00Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	 BASE is free software; you can redistribute it and/or
	 modify it under the terms of the GNU General Public License
	 as published by the Free Software Foundation; either version 3
	 of the License, or (at your option) any later version.
	
	 BASE is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.
	
	 You should have received a copy of the GNU General Public License
	 along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import java.util.Set;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.FileType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemProxy;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PlatformVariant;
import net.sf.basedb.core.SystemItems;

public class TestDataFileType
{

	static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing data file types");
		write_header();
		// Standard tests: create, load and list
		test_load(SystemItems.getId("affymetrix.cel"));
		
		int id1 = test_create("Test file type", "test.filetype", Item.RAWBIOASSAY, SystemItems.getId(FileType.RAW_DATA));
		test_list();

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id1);
		write("++Testing data file types "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String name, String externalId, Item itemType, int genericTypeId)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.DATAFILETYPE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			FileType genericType = genericTypeId == 0 ? null : FileType.getById(dc, genericTypeId);
			DataFileType type = DataFileType.getNew(dc, externalId, itemType);
			type.setGenericType(genericType);
			type.setName(name);
			dc.saveItem(type);
			dc.commit();
			id = type.getId();
			write_item(0, type);
			write("--Create data file type OK");
		}
		catch (Throwable ex)
		{
			write("--Create data file type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			DataFileType type = DataFileType.getById(dc, id);
			write_item(0, type);
			write("--Load data file type OK");
		}
		catch (Throwable ex)
		{
			write("--Load data file type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<DataFileType> l = DataFileType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List data file types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List data file types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			DataFileType l = DataFileType.getById(dc, id);
			dc.deleteItem(l);
			Set<ItemProxy> using = l.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(l + " is used by " + using);
			}
			dc.commit();
			write("--Delete data file type OK");
		}
		catch (Throwable ex)
		{
			write("--Delete data file type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\\tExternal ID");
			write("-- \t-- \t--------- \t-----------\t---------");
		}
	}
	
	static void write_item(int i, DataFileType type)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+type.getId()+"\t"+type.getName()+"\t"+
					type.getDescription() + "\t" + type.getExternalId());
		}
	}
	
	static void write_item(int i, PlatformVariant v)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+v.getId()+"\t"+v.getName()+"\t"+
					v.getDescription() + "\t" + v.getExternalId() + "\t" + v.isFileOnly() + "\t" +
					v.getRawDataType() + "\t" + v.getPlatform());
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}

	static int get_id_for_external(String externalId)
	{
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			DataFileType type = DataFileType.getByExternalId(dc, externalId);
			id = type.getId();
			write("--Get data file type (" + externalId + ") OK");
		}
		catch (Throwable ex)
		{
			write("--Get data file type (" + externalId + ") FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;		
	}

}
