/*
	$Id: TestDirty.java 5319 2010-04-20 08:02:16Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import net.sf.basedb.core.query.Aggregations;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Selects;
import net.sf.basedb.core.query.SqlResult;

public class TestDirty
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing import of dirty data");
		boolean caseInsensitive = HibernateUtil.getDbEngine().caseInsensitiveComparison();
		write("--case insensitive = " + caseInsensitive);
		
		// Tests for checking case sensitivity and white space truncation
		TestReporter.test_import_from_file("data/test.import.dirty.txt", "\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", "\\t", 4, 3);

		// If DB is case sensitive the two reporters should be different
		int id1 = TestReporter.test_externalId("demoblank");
		int id2= TestReporter.test_externalId("demoBLANK");
		if (caseInsensitive)
		{
			write("--demoblank == demoBLANK " + (id1 == id2 ? "OK" : "FAILED"));
			ok = ok && (id1 == id2);
		}
		else
		{
			write("--demoblank != demoBLANK " + (id1 != id2 ? "OK" : "FAILED"));
			ok = ok && (id1 != id2);
		}
		
		// Check that white space has been trimmed
		int id3 = TestReporter.test_externalId("demoB");
		int id4 = TestReporter.test_externalId("demoB ");
		write("--demoB == demoB<space> " + (id3 == id4 ? "OK" : "FAILED"));
		ok = ok && (id3 == id4);
		
		ok = ok && TestReporter.test_list(caseInsensitive ? 3 : 4);
		
		// Test import raw data
		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, false);
		TestArrayDesign.test_import_from_file(arrayDesignId, "data/test.import.dirty.txt", 
				"\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", null, 0, 2, 1, 4);
		
		int rawBioAssayId = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", null, 0, 0, 0, arrayDesignId, 1, false);
		int fileId = TestFile.test_create("data/test.import.dirty.txt", false, false);
		
		int pluginDefinitionId = TestPluginDefinition.test_get("net.sf.basedb.plugins.RawDataFlatFileImporter");
		int pluginConfigurationId = TestRawDataFlatFileImporter.test_create_configuration(pluginDefinitionId);
		int jobId = TestRawDataFlatFileImporter.test_create_job(pluginConfigurationId, rawBioAssayId, fileId);
		ok = ok && TestJob.test_execute(jobId, false);
		ok = ok && TestRawBioAssay.test_list_raw_data(rawBioAssayId, 0, 0, 5, false);
		test_sum_count(rawBioAssayId, "x", 9920f, 5);
		test_sum_count(rawBioAssayId, "y", 22930f, 4);
		test_sum_count(rawBioAssayId, "diameter", 240f, 4);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestRawBioAssay.test_delete(rawBioAssayId);
		TestArrayDesign.test_delete(arrayDesignId);
		TestJob.test_delete(jobId);
		TestPluginConfiguration.test_delete(pluginConfigurationId);
		TestFile.test_delete(fileId);
		TestReporter.test_delete();
		write("++Testing import of dirty data "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_sum_count(int rawBioAssayId, String property, float expectedSum, int expectedCount)
	{
		if (rawBioAssayId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Expression rawExpression = Dynamic.rawData(property);
			RawBioAssay rba = RawBioAssay.getById(dc, rawBioAssayId);
			DynamicRawDataQuery query = rba.getDynamicRawData();
			query.select(Selects.expression(Aggregations.sum(rawExpression), "s"));
			query.select(Selects.expression(Aggregations.count(rawExpression, false), "c"));
			
			DynamicResultIterator iterator = query.iterate(dc);
			SqlResult result = iterator.next();
			
			float sum = result.getFloat(1);
			int count = result.getInt(2);
			
			if (Math.abs(expectedSum - sum) > 0.1)
			{
				throw new Exception("Unexpected sum for '"+property+"': " + sum +"; expected " +expectedSum);
			}
			if (expectedCount != count)
			{
				throw new Exception("Unexpected count for '"+property+"': " + count +"; expected " +expectedCount);
			}
			write("--Test sum and count for '" + property + "' OK (sum=" + sum + "; count="+ count +")");
		}
		catch (Throwable ex)
		{
			write("--Test sum and count FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
}

