/*
 $Id: TestGalExporter.java 5319 2010-04-20 08:02:16Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg
 
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.PrintWriter;
import java.util.Arrays;

import org.apache.commons.io.output.NullOutputStream;

import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.plugins.GalExporter;
import net.sf.basedb.util.ConsoleProgressReporter;

/**
	@base.modified $Date: 2010-04-20 10:02:16 +0200 (Tue, 20 Apr 2010) $
	@author Nicklas
	@version 2.7
*/
public class TestGalExporter
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing GAL export");

		// Import reporters
		TestReporter.test_import_from_file("data/test.rawdata.import.txt", "\"Block\"\\t\"Column\"\\t\"Row\"\\t\"Name\"\\t\"ID\".*", "\\t", 4, 3);
		
		// Create ArrayDesign needed for print map import
		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, false);
		int plateGeometryId = TestPlateGeometry.test_create(16, 24, false);
		int plateTypeId = TestPlateType.test_create(plateGeometryId, false);
		int plate384 = TestPlate.test_create(plateTypeId, true);
		int[] plates = new int[4];
		Arrays.fill(plates, plate384);
		TestArrayDesign.test_add_plates(arrayDesignId, plates);

		// Print map import
		int fileId = TestFile.test_create("data/test.parse.printmap.tam", false, false);
		int printMapPluginDefinitionId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.PrintMapFlatFileImporter");
		int jobId = TestPrintMapFlatFileImporter.test_create_job_printmap(printMapPluginDefinitionId, fileId, arrayDesignId);
		ok &= TestJob.test_execute(jobId, false);
		
		// Export as GAL
		test_gal_export(arrayDesignId);

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		TestArrayDesign.test_delete(arrayDesignId);
		TestJob.test_delete(jobId);
		TestPlate.test_delete(plate384);
		TestPlateType.test_delete(plateTypeId);
		TestPlateGeometry.test_delete(plateGeometryId);
		TestReporter.test_delete();
		TestFile.test_delete(fileId);

		write("++Testing GAL export "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	
	private static void test_gal_export(int arrayDesignId)
	{
		if (arrayDesignId == 0) return;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			ProgressReporter progress = null;
			PrintWriter writer = null;
			if (!TestUtil.getSilent()) 
			{
				writer = new PrintWriter(System.out);
				progress = new ConsoleProgressReporter();
			}
			else
			{
				writer = new PrintWriter(new NullOutputStream());
			}
			ArrayDesign design = ArrayDesign.getById(dc, arrayDesignId);
			GalExporter exporter = new GalExporter();
			exporter.exportGal(dc, design, null, writer, progress);
			write("--GAL export OK");
		}
		catch (Throwable ex)
		{
			write("--GAL export FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write(String message)
	{
		System.out.println(message);
	}

}
